-- EventTimer.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["Event Timer"]))
end

local ASHRAN_LANDMARK_ID = 42;

local EventTimer = AshranBuddy:RegisterModule("EventTimer")
LibStub("AceTimer-3.0"):Embed(EventTimer)

local TimerState = {
	Hidden = -1,
	Inactive = 0,
	Intermission = 1,
	Announced = 2,
	PreActive = 3,
	Active = 4
}

-- Inherited
function EventTimer:OnEnable()
	self:RegisterEvent("CHAT_MSG_MONSTER_EMOTE", "OnChatMessageMonsterEmote")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChangedNewArea")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	self:RegisterEvent("GROUP_JOINED", "OnGroupJoined")
	
	self:CreateContainerFrame(220, 65)
	self:Migrate()
	
	if (not AshranBuddy.db.profile.EventTimer.locked) then
		EventTimer:Unlock()
	end
	
	self:SendRequest()
end

-- Inherited
function EventTimer:OnDisable()
	self:UnregisterAllEvents()
	self.containerFrame:Hide()
	
	for i = 1, #EventTimer.statusBars do
		local statusBar = self.statusBars[i]
		self:OnStateChange(statusBar, TimerState.Hidden)
	end
end

function EventTimer:OnChatMessageMonsterEmote(event, message, sender, language, channelString, target, flags, unused1, channelNumber, channelName, unused2, counter)
	if (AshranBuddy:IsInAshran()) then
		for i = 1, #self.statusBars do
			local statusBar = self.statusBars[i]
			if (message:find(statusBar.eventId)) then
				self:OnStateChange(statusBar, TimerState.Announced)
			end
		end
	end
end

function EventTimer:OnZoneChangedNewArea(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
		self:SendRequest()
	else
		if (AshranBuddy.db.profile.EventTimer.locked == false) then
			self:Unlock()
		else
			self.containerFrame:Hide()
		end
	end
end

function EventTimer:OnPlayerEnteringWorld(event)
	if (AshranBuddy:IsInAshran()) then
		self.containerFrame:Show()
		self:SendRequest()
	else
		if (AshranBuddy.db.profile.EventTimer.locked == false) then
			self:Unlock()
		else
			self.containerFrame:Hide()
		end
	end
end

function EventTimer:OnGroupJoined(event)
	if (AshranBuddy:IsInAshran()) then
		self:SendRequest()
	end
end

function EventTimer:OnUpdate()
	if (not AshranBuddy:IsInAshran()) then
		return
	end
	
	for i = 1, #EventTimer.statusBars do
		local statusBar = EventTimer.statusBars[i]
		local value = statusBar:GetValue()
		local minValue, maxValue = statusBar:GetMinMaxValues()
		local exists = EventTimer:FindWorldMapLandmark(statusBar.eventName, statusBar.mapX, statusBar.mapY)

		if (exists) then
			EventTimer:OnStateChange(statusBar, TimerState.Active)
		elseif (not exists and statusBar.state == TimerState.Active) then
			EventTimer:OnStateChange(statusBar, TimerState.Inactive)
		end
	end
	
	if (EventTimer.statusBarsDirty and AshranBuddy.db.profile.EventTimer.stacked) then
		local frame = EventTimer.containerFrame
		local bars = { }
		for i = 1, #EventTimer.statusBars do
			bars[#bars + 1] = { EventTimer.statusBars[i]:GetOrder(), EventTimer.statusBars[i] }
		end
		table.sort(bars, compare)
		
		local y = -5
		for key, value in pairs(bars) do
			value[2]:ClearAllPoints()
			value[2]:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, y)
			y = y - 15
		end
	
		EventTimer.statusBarsDirty = false
	end
end

function compare(a, b)
  return a[1] < b[1]
end

function EventTimer:OnMouseDown()
	if (AshranBuddy.db.profile.EventTimer.locked == false) then
		self:StartMoving()
	end
end

function EventTimer:OnMouseUp()
	if (AshranBuddy.db.profile.EventTimer.locked == false) then
		self:StopMovingOrSizing()
	end
end

function EventTimer:OnStatusBarClick(button)
	local value = self:GetValue()
	local minValue, maxValue = self:GetMinMaxValues()
	local timeLeft = maxValue - value
	local timeText = string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60)
	local message = string.format(L["AshranBuddy: [Event: %s] "], self.eventName)
	
	if (self.state == TimerState.Inactive) then
		message = string.format(L["%s is on cooldown for %s."], message, timeText)
	elseif (self.state == TimerState.Intermission) then
		message = string.format(L["%s will announce randomly within the next %s."], message, timeText)
	elseif (self.state == TimerState.Announced) then
		message = string.format(L["%s will begin in %s."], message, timeText)
	elseif (self.state == TimerState.Active) then
		message = string.format(L["%s is currently active."], message)
	end

	if (button == "LeftButton" and IsShiftKeyDown() and IsInRaid()) then
		SendChatMessage(message, "RAID")
	end
	if (button == "LeftButton" and IsAltKeyDown() and IsInRaid()) then
		SendChatMessage(message , "RAID_WARNING")
	end
	if (button == "LeftButton" and IsControlKeyDown()) then
		local index = GetChannelName(L["General - Ashran"])
		if (index ~= nil) then 
			SendChatMessage(message , "CHANNEL", nil, index)
		end
	end
end

function EventTimer:OnStateChange(statusBar, state)
	statusBar.state = state
	if (statusBar.eventTimer ~= nil) then
		self:CancelTimer(statusBar.eventTimer)
		statusBar.eventTimer = nil
	end
	if (state == TimerState.Hidden) then
		statusBar:Hide()
	end
	if (AshranBuddy:IsInAshran()) then	
		if (state == TimerState.Inactive) then
			self:SetStatusBarFrame(statusBar, 0, 0, 1800, 0.5, 0.5, 0.5)
			statusBar.eventTimer = EventTimer:ScheduleRepeatingTimer("OnTimerCallback", 1, statusBar, TimerState.Intermission)
		elseif (state == TimerState.Intermission) then
			self:SetStatusBarFrame(statusBar, 0, 0, 600, 1, 0, 0)
			statusBar.eventTimer = EventTimer:ScheduleRepeatingTimer("OnTimerCallback", 1, statusBar, TimerState.Hidden)
		elseif (state == TimerState.Announced) then
			self:SetStatusBarFrame(statusBar, 0, 0, 180, 0.26, 0.42, 1)
			statusBar.eventTimer = EventTimer:ScheduleRepeatingTimer("OnTimerCallback", 1, statusBar, TimerState.PreActive)
		elseif (state == TimerState.PreActive) then
			self:SetStatusBarFrame(statusBar, 60, 0, 60, 0, 1, 0, "")
		elseif (state == TimerState.Active) then
			self:SetStatusBarFrame(statusBar, 60, 0, 60, 0, 1, 0, "")
		end
	end
	self.statusBarsDirty = true
end

function EventTimer:OnTimerCallback(statusBar, changeTo)
	local value = statusBar:GetValue()
	local minValue, maxValue = statusBar:GetMinMaxValues()
	local timeLeft = maxValue - value
	
	statusBar:SetEventTime(string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60))
	statusBar:SetValue(value + 1)

	if (value >= maxValue) then
		if (statusBar.eventName == L["Ogre Fires"] and changeTo == TimerState.PreActive) then
			-- TODO: Ogre Fires cannot be tracked correctly due to a Blizzard issue
			self:OnStateChange(statusBar, TimerState.Inactive)
		else
			self:OnStateChange(statusBar, changeTo)
		end
	end
end

function EventTimer:Lock()
	if (not AshranBuddy:IsInAshran()) then
		self.containerFrame:Hide()
	end
	self.handle:Hide()
	
	local point, relativeTo, relativePoint, x, y = self.containerFrame:GetPoint(1)
	AshranBuddy.db.profile.EventTimer.point = point
	AshranBuddy.db.profile.EventTimer.relativePoint = relativePoint
	AshranBuddy.db.profile.EventTimer.x = x
	AshranBuddy.db.profile.EventTimer.y = y
end

function EventTimer:Unlock()
	self.containerFrame:Show()
	self.handle:Show()
end

function EventTimer:CreateContainerFrame(width, height)
	local frame = self.containerFrame
	if (frame.initialized) then
		self.containerFrame:Show()
		return
	end

	self.statusBars = { }
	self.statusBarsDirty = false
	
	local profile = AshranBuddy.db.profile.EventTimer
	
	frame:SetParent(UIParent)
	frame:SetSize(width, height)
	frame:SetPoint(profile.point, UIParent, profile.relativePoint, profile.x, profile.y)
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetBackdrop({ bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background" })
	frame:SetBackdropColor(0, 0, 0, 0)
	frame:SetToplevel(true)
	
	local handle = CreateFrame("Frame", nil, frame)
	handle:SetSize(width, height)
	handle:SetPoint("TOPLEFT", frame)
	handle:Hide()

	local handleBox = handle:CreateTexture(nil, "OVERLAY", nil, -1)
	handleBox:SetTexture(0, 0, 0, 0.6)
	handleBox:SetSize(width, height)
	handleBox:SetPoint("TOPLEFT")
	
	local handleFontString = handle:CreateFontString("HandleLabel", "OVERLAY", "TextStatusBarText")
	handleFontString:SetPoint("CENTER")
	handleFontString:SetText(L["Ashran Buddy\n\nYou MUST Type /ab lock or\n/ashranbuddy lock to activate.\nBars won't show until event is announced"])

	-- Children
	self.handle = handle
	self.statusBars[1] = self:CreateStatusBarFrame("178019", L["Empowered Ore"], L["Collect Empowered Ore"],  round(0.545, 2), round(0.690, 2), frame, -5)
	self.statusBars[2] = self:CreateStatusBarFrame("178021", L["Ogre Fires"], L["Ogre Fires"], 0, 0, frame, -20)
	self.statusBars[3] = self:CreateStatusBarFrame("178020", L["Risen Spirits"], L["Risen Spirits"], round(0.372, 2), round(0.631, 2), frame, -35)
	self.statusBars[4] = self:CreateStatusBarFrame("178022", L["Stadium Racing"], L["Stadium Racing"], round(0.376, 2), round(0.375, 2), frame, -50)

	-- Events
	frame:SetScript("OnUpdate", EventTimer.OnUpdate)
	frame:SetScript("OnMouseDown", EventTimer.OnMouseDown)
	frame:SetScript("OnMouseUp", EventTimer.OnMouseUp)
	
	frame.initialized = true
end

function EventTimer:CreateStatusBarFrame(id, title, eventName, mapX, mapY, parent, y)
	local statusBar = CreateFrame("StatusBar", title, parent)
	statusBar:SetWidth(200)
	statusBar:SetHeight(10)
	statusBar:SetPoint("TOPLEFT", parent, "TOPLEFT", 5, y)
	statusBar:SetMinMaxValues(0, 180)
	statusBar:SetValue(0)
	statusBar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar")
	statusBar:SetStatusBarColor(0.26, 0.42, 1)

	-- Artwork Layer
	local borderLeft = statusBar:CreateTexture("BorderLeft", "ARTWORK")
	borderLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderLeft:SetTexCoord(0.007843, 0.043137, 0.193548, 0.774193)
	borderLeft:SetPoint("LEFT", -3, 0)
	borderLeft:SetSize(9, 14)

	local borderRight = statusBar:CreateTexture("BorderRight", "ARTWORK")
	borderRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderRight:SetTexCoord(0.043137, 0.007843, 0.193548, 0.774193)
	borderRight:SetPoint("RIGHT", 3, 0)
	borderRight:SetSize(9, 14)
	
	local borderMid = statusBar:CreateTexture("BorderMid", "ARTWORK")
	borderMid:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-BarBorder")
	borderMid:SetTexCoord(0.113726, 0.1490196, 0.193548, 0.774193)
	borderMid:SetPoint("TOPLEFT", borderLeft, "TOPRIGHT")
	borderMid:SetPoint("BOTTOMRIGHT", borderRight, "BOTTOMLEFT")
	borderMid:SetSize(9, 14)

	local eventNameFontString = statusBar:CreateFontString("EventNameLabel", "ARTWORK", "TextStatusBarText")
	eventNameFontString:SetPoint("LEFT", statusBar, "LEFT", 8, 0)

	local eventTimeFontString = statusBar:CreateFontString("EventTimeLabel", "ARTWORK", "TextStatusBarText")
	eventTimeFontString:SetPoint("RIGHT")
	
	if (not AshranBuddy.db.profile.EventTimer.showTimer) then
		eventTimeFontString:Hide()
	end
	
	-- Background Layer
	local barFull = statusBar:CreateTexture("BarFull", "BACKGROUND")
	barFull:SetVertexColor(0.04, 0.07, 0.18)
	
	statusBar.eventId = id
	statusBar.eventName = eventName
	statusBar.eventNameFontString = eventNameFontString
	statusBar.eventTimeFontString = eventTimeFontString
	statusBar.state = TimerState.Hidden
	statusBar.mapX = mapX
	statusBar.mapY = mapY
	
	statusBar.GetEventId = function(self)
		return self.eventId
	end
	statusBar.GetEventName = function(self)
		return self.eventName
	end
	statusBar.GetEventTitle = function(self)
		return self.eventNameFontString:GetText()
	end
	statusBar.SetEventTitle = function(self, value)
		self.eventNameFontString:SetText(value)
	end
	statusBar.GetEventTime = function(self)
		return self.eventTimeFontString:GetText()
	end
	statusBar.SetEventTime = function(self, value)
		self.eventTimeFontString:SetText(value)
	end
	statusBar.GetOrder = function(self)
		local value = self:GetValue()
		local category = 6
		if (self.state == TimerState.Inactive) then
			category = 5
		elseif (self.state == TimerState.Intermission) then
			category = 4
		elseif (self.state == TimerState.Announced) then
			category = 3
		elseif (self.state == TimerState.PreActive) then
			category = 2
		elseif (self.state == TimerState.Active) then
			category = 1
		end
		category = category * 100000
		return (category - value)
	end

	statusBar:SetScript("OnMouseDown", EventTimer.OnStatusBarClick)
	
	statusBar:SetEventTitle(title)
	statusBar:SetEventTime("")
	statusBar:Hide()
	
	return statusBar
end

function EventTimer:SetStatusBarFrame(statusBar, value, minValue, maxValue, r, g, b, text)
	local timeLeft = maxValue - value
	if (text == nil) then
		local text = string.format(L["%.2d:%.2d"], timeLeft / 60 % 60, timeLeft % 60)
	end
	
	statusBar:SetEventTime(text)
	statusBar:SetMinMaxValues(minValue, maxValue)
	statusBar:SetValue(value)
	statusBar:SetStatusBarColor(r, g, b)
	statusBar:Show()
end

function EventTimer:FindWorldMapLandmark(eventName, mapX, mapY) 
	local count = GetNumMapLandmarks()
	for i = 1, count do
        local worldMapPOIName = "WorldMapFramePOI" .. i
        local worldMapPOI = _G[worldMapPOIName]
		
		local name, description, textureIndex, x, y, mapLinkID, inBattleMap, graveyardID, areaID, poiID, isObjectIcon = GetMapLandmarkInfo(i)
		if (textureIndex == ASHRAN_LANDMARK_ID and mapX == round(x, 2) and mapY == round(y, 2)) then
			return true
		end
	end
	return false
end

-- Inherited
function EventTimer:OnChatCommand(input)
	if (input == "unlock") then
		EventTimer:Unlock()
		AshranBuddy.db.profile.EventTimer.locked = false
	elseif (input == "lock") then
		EventTimer:Lock()
		AshranBuddy.db.profile.EventTimer.locked = true
	elseif (input == "sync") then
		self:SendRequest()
	end
end

-- Inherited
function EventTimer:OnCreateOptions(options)
	options.EventTimer = {
		name = L["Event Timer"],
		type = "group",
		args = {
			header = {
				name = L["Event Timer"],
				type = "header",
				order = 0
			},
			headerDescription = {
				name = L["Tracks and synchronizes timers for all Ashran events."],
				type = "description",
				order = 1
			},
			enabled = {
				name = L["Enabled"],
				desc = L["Enables the Ashran Event Timer module."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.enabled = val; if (val) then self:Enable() else self:Disable() end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.enabled end,
				order = 2
			},
			enabledDescription = {
				name = L["Enables the Ashran Event Timer module."],
				type = "description",
				order = 3
			},
			locked = {
				name = L["Lock Event Timer"],
				desc = L["Locks and unlocks the event timer frame."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.locked = val; if (AshranBuddy.db.profile.EventTimer.locked) then self:Lock() else self:Unlock() end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.locked end,
				order = 4
			},
			lockedDescription = {
				name = L["Locks and unlocks the event timer frame."],
				type = "description",
				order = 5
			},
			showTimer = {
				name = L["Show Numbered Timer"],
				desc = L["Displays numbered timer on the right of the countdown bar."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.showTimer = val; if (AshranBuddy.db.profile.EventTimer.showTimer) then for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Show() end else for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Hide() end end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.showTimer end,
				order = 6
			},
			showTimerDescription = {
				name = L["Displays numbered timer on the right of the countdown bar."],
				type = "description",
				order = 7
			},
			stacked = {
				name = L["Stack Timers"],
				desc = L["Neatly stack and sort the timers."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.EventTimer.showTimer = val; if (AshranBuddy.db.profile.EventTimer.showTimer) then for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Show() end else for i = 1, #self.statusBars do self.statusBars[i].eventTimeFontString:Hide() end end end,
				get = function(info) return AshranBuddy.db.profile.EventTimer.showTimer end,
				order = 8
			},
			stackedDescription = {
				name = L["Neatly stack and sort the timers."],
				type = "description",
				order = 9
			},
		},
	}
end

function EventTimer:OnMessageReceived(packet, distribution, sender)
	-- A raid member has requested a synchronise
	if (packet.type == "request") then
		EventTimer:CommRequest(sender)
	end
	
	-- You requested a synchronise and here it is
	if (packet.type == "synchronise" and distribution == "WHISPER") then
		EventTimer:CommSynchronise(sender, packet.data)
	end
end

function EventTimer:SendRequest()
	if (IsInRaid()) then
		local packet = { type = "request", data = {} }
		self:SendMessage(packet, "RAID")
	end
end

function EventTimer:CommRequest(sender)
	if (EventTimer.enabled) then
		local packet = { type = "synchronise", data = { } }
		for i = 1, #EventTimer.statusBars do
			local statusBar = EventTimer.statusBars[i]
			packet.data[i] = { state = statusBar.state, value = statusBar:GetValue() }
		end

		EventTimer:SendMessage(packet, "WHISPER", sender)
	end
end

function EventTimer:CommSynchronise(sender, data)
	if (data ~= nil) then
		for i = 1, #EventTimer.statusBars do
			if (data[i].state ~= TimerState.Hidden) then
				local statusBar = EventTimer.statusBars[i]
				EventTimer:OnStateChange(statusBar, data[i].state)
				statusBar:SetValue(data[i].value)
			end
		end
	end
end

function EventTimer:Migrate()
	if (AshranBuddy.db.profile.locked ~= nil) then
		local profile = AshranBuddy.db.profile
		
		profile.EventTimer.enabled = self.enabled
		profile.EventTimer.locked = AshranBuddy.db.profile.locked
		profile.EventTimer.showTimer = AshranBuddy.db.profile.showTimer
		profile.EventTimer.point = AshranBuddy.db.profile.point
		profile.EventTimer.relativePoint = AshranBuddy.db.profile.relativePoint
		profile.EventTimer.x = AshranBuddy.db.profile.x
		profile.EventTimer.y = AshranBuddy.db.profile.y
		
		profile.locked = nil
		profile.showTimer = nil
		profile.point = nil
		profile.relativePoint = nil
		profile.x = nil
		profile.y = nil
	end
end

function round(num, precision)
	return tonumber(string.format("%." .. (precision or 0) .. "f", num))
end