﻿-- NemesisFinder.lua

local L = LibStub("AceLocale-3.0"):GetLocale("AshranBuddy")

local AshranBuddy = _G.AshranBuddy
if (not AshranBuddy) then
	DEFAULT_CHAT_FRAME:AddMessage(format(L["Module '%s' requires AshranBuddy"], L["Nemesis Finder"]))
end

local MAX_RAID_MEMBERS = 40
local NEMESIS_QUESTS = {
	36964, --Nemesis: Orcslayer"
	36921, --Nemesis: Manslayer"
	36960, --Nemesis: Death Stalker"
	36932, --Nemesis: Scourge of the Kaldorei"
	36958, --Nemesis: Slayer of Sindorei"
	36962, --Nemesis: The Butcher"
	36966, --Nemesis: Troll Hunter"
	36968, --Nemesis: Huojins Fall"
	36934, --Nemesis: Terror of the Tushui"
	36930, --Nemesis: Draenei Destroyer"
	36928, --Nemesis: Worgen Hunter"
	36926, --Nemesis: Gnomebane"
	36970, --Nemesis: Killer of Kezan"
	36924  --Nemesis: Dwarfstalker"
}

local NemesisFinder = AshranBuddy:RegisterModule("NemesisFinder")
NemesisFinder.quest = nil
NemesisFinder.buttons = { }
NemesisFinder.members = { }

function NemesisFinder:OnEnable()
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "OnGroupRosterUpdate")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld")
	
	self:Migrate()
	
	self:OnGroupRosterUpdate()
end

function NemesisFinder:OnDisable()
	self:UnregisterAllEvents()
	self:RemoveHighlight()
end

function NemesisFinder:Initialize()
	if (self.quest == nil) then
		self.quest = self:GetNemesisQuest()
	end
	if (#self.buttons == 0) then
		for i = 1, MAX_RAID_MEMBERS do
			self.buttons[i] = _G["RaidGroupButton"..i]
		end
	end
end

function NemesisFinder:OnGroupRosterUpdate()
	self:Initialize()
	if (self.quest ~= nil) then
		-- Reset Nemesis highlight
		self:RemoveHighlight()
		self:SendRequest()
	end
end

function NemesisFinder:OnPlayerEnteringWorld()
	self:OnGroupRosterUpdate()
end

function NemesisFinder:AddHighlight(name)
	if (name ~= nil) then
		local found = false
		for i = 1, GetNumGroupMembers() do
			if (self.buttons[i].name ~= nil) then
				if (string.find(self.buttons[i].name, name)) then
					self.buttons[i]:LockHighlight()
					break
				end
			end
		end
	end
end

function NemesisFinder:RemoveHighlight()
	for i = 1, GetNumGroupMembers() do
		if (self.buttons[i] ~= nil and self.buttons[i].name ~= nil) then
			if (not self:FindMember(self.buttons[i].name)) then
				self.buttons[i]:UnlockHighlight()
			end
		end
	end
end

function NemesisFinder:FindMember(name)
	local found = false
	for i = 1, #NemesisFinder.members do
		if (string.find(NemesisFinder.members[i], name)) then
			found = true
			break
		end
	end
	return found
end

function NemesisFinder:OnChatCommand(input)
end

function NemesisFinder:OnCreateOptions(options)
	options.NemesisFinder = {
		name = L["Nemesis Finder"],
		type = "group",
		args = {
			header = {
				name = L["Nemesis Finder"],
				type = "header",
				order = 0
			},
			headerDescription = {
				name = L["Highlights all players in your current raid group that share the same Nemesis Quest as you. (Requires other players to have AshranBuddy)"],
				type = "description",
				order = 1
			},
			enabled = {
				name = L["Enabled"],
				desc = L["Enables the Nemesis Finder module."],
				type = "toggle",
				set = function(info, val) AshranBuddy.db.profile.NemesisFinder.enabled = val; if (val) then self:Enable() else self:Disable() end end,
				get = function(info) return AshranBuddy.db.profile.NemesisFinder.enabled end,
				order = 2
			},
			enabledDescription = {
				name = L["Enables the Nemesis Finder module."],
				type = "description",
				order = 3
			},
		},
	}
end

function NemesisFinder:GetNemesisQuest()
	local nemesisQuest = nil
	local count = GetNumQuestLogEntries()
	for i = 1, count do
        local _, _, _, _, _, _, _, quest, _, _, _, _, _, _ = GetQuestLogTitle(i)
		for j = 1, #NEMESIS_QUESTS do
			if (NEMESIS_QUESTS[j] == quest) then
				nemesisQuest = quest
				break
			end
		end
		if (nemesisQuest ~= nil) then
			break
		end
	end
	return nemesisQuest
end

function NemesisFinder:OnMessageReceived(packet, distribution, sender)
	-- A raid member has requested a synchronise
	if (packet.type == "request") then
		NemesisFinder:CommRequest(sender, packet.data.quest)
	end
	
	-- You requested a synchronise and here it is
	if (packet.type == "synchronise" and distribution == "WHISPER") then
		NemesisFinder:CommSynchronise(sender)
	end
end

function NemesisFinder:SendRequest()
	if (IsInRaid()) then
		local packet = { type = "request", data = { quest = self.quest } }
		-- Send the nemesis quest that I am currently on
		self:SendMessage(packet, "RAID")
	end
end

function NemesisFinder:CommRequest(sender, quest)
	local packet = { type = "synchronise", data = { } }
	-- If I have the same nemesis quest then respond otherwise ignore
	if (self.quest == quest) then
		NemesisFinder:SendMessage(packet, "WHISPER", sender)
	end
end

function NemesisFinder:CommSynchronise(sender)
	NemesisFinder:AddHighlight(sender)
	table.insert(NemesisFinder.members, sender)
end

function NemesisFinder:Migrate()
	if (AshranBuddy.db.profile.NemesisFinder == nil) then
		local profile = AshranBuddy.db.profile
		profile.NemesisFinder = { }
		profile.NemesisFinder.enabled = true
	end
end
